(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? module.exports = factory() :
  typeof define === 'function' && define.amd ? define(factory) :
  (global = typeof globalThis !== 'undefined' ? globalThis : global || self, global.tailwindGroupVariant = factory());
})(this, (function () { 'use strict';

  /* eslint-disable complexity */

  /**
   * @name tailwind-group-variant
   * @license MIT license.
   * @copyright (c) 2022 Christian Schurr
   * @author Christian Schurr <fischerc.it@gmail.com>
   */
  const whitespaceChars = [' ', '\n', '\t', '\r', '\f', '\v', '\u00a0', '\u1680', '\u2000', '\u200a', '\u2028', '\u2029', '\u202f', '\u205f', '\u3000', '\ufeff'];
  const badChars = ['"', "'", '`', '\\', '[', '\n', '\r'];
  const State = {
    init: 'init',
    variantOrWord: 'variantOrWord',
    variant: 'variant',
    'stack.Init': 'stack.Init',
    'stack.VariantOrWord': 'stack.VariantOrWord',
    'stack.Variant': 'stack.Variant',
    'stack.Close': 'stack.Close'
  };

  function stackClose(machine, idx, hasWord) {
    // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
    const currentStack = machine.context.stack.pop();

    if (hasWord) {
      currentStack.matches.push(machine.context.content.substring(machine.context.variantStartIdx, idx));
    }

    if (machine.context.stack.length > 0) {
      var _machine$context$stac;

      (_machine$context$stac = machine.context.stack[machine.context.stack.length - 1]) == null ? void 0 : _machine$context$stac.nestedMatches.push({ ...currentStack,
        endIdx: idx
      });
      machine.state = State['stack.Close'];
    } else {
      currentStack.matches.push(...currentStack.nestedMatches.flatMap(nestedStack => nestedStack.matches.map(match => `${nestedStack.variant}${match}`)));
      machine.context.matches.push({
        start: currentStack.startIdx,
        end: idx,
        content: currentStack.matches.map(match => `${currentStack.variant}${match}`).join(' ')
      });
      machine.state = State.init;
    }
  }

  function stackError(machine) {
    for (const stack of machine.context.stack) {
      if (stack.nestedMatches.length) {
        for (const nestedStack of stack.nestedMatches) {
          machine.context.matches.push({
            start: nestedStack.startIdx,
            end: nestedStack.endIdx,
            content: nestedStack.matches.map(match => `${nestedStack.variant}${match}`).join(' ')
          });
        }
      }
    }

    machine.context.stack.length = 0;
    machine.state = State.init;
  }

  function openStack(machine) {
    machine.state = State['stack.Init'];
    machine.context.stack.push({
      matches: [],
      nestedMatches: [],
      variant: machine.context.content.substring(machine.context.variantStartIdx, machine.context.variantEndIdx + 1),
      startIdx: machine.context.variantStartIdx
    });
  }

  const transformMachine = {
    [State.init]: (idx, char, machineState) => {
      if (!machineState.settings.badChars.includes(char)) {
        machineState.context.variantStartIdx = idx;
        machineState.state = State.variantOrWord;
      }
    },
    [State.variantOrWord]: (idx, char, machineState) => {
      if (char === machineState.settings.variantChar) {
        machineState.context.variantEndIdx = idx;
        machineState.state = State.variant;
      } else if (machineState.settings.badChars.includes(char)) {
        machineState.state = State.init;
      }
    },
    [State.variant]: (_idx, char, machineState) => {
      if (char === machineState.settings.expandOpen) {
        openStack(machineState);
      } else if (machineState.settings.badChars.includes(char)) {
        machineState.state = State.init;
      } else {
        machineState.state = State.variantOrWord;
      }
    },
    'stack.Init': (idx, char, machineState) => {
      if (machineState.settings.badChars.includes(char)) {
        stackError(machineState);
      } else if (char === machineState.settings.expandClose) {
        stackError(machineState);
      } else {
        machineState.context.variantStartIdx = idx;
        machineState.state = State['stack.VariantOrWord'];
      }
    },
    'stack.VariantOrWord': (idx, char, machineState) => {
      if (char === machineState.settings.separatorChar) {
        var _machineState$context;

        (_machineState$context = machineState.context.stack[machineState.context.stack.length - 1]) == null ? void 0 : _machineState$context.matches.push(machineState.context.content.substring(machineState.context.variantStartIdx, idx));
        machineState.state = State['stack.Init'];
      } else if (char === machineState.settings.variantChar) {
        machineState.context.variantEndIdx = idx;
        machineState.state = State['stack.Variant'];
      } else if (char === machineState.settings.expandClose) {
        stackClose(machineState, idx, true);
      }
    },
    'stack.Variant': (_idx, char, machineState) => {
      if (char === machineState.settings.expandOpen) {
        openStack(machineState);
      } else if (machineState.settings.badChars.includes(char)) {
        stackError(machineState);
      } else {
        machineState.state = State['stack.VariantOrWord'];
      }
    },
    'stack.Close': (idx, char, machineState) => {
      if (machineState.settings.badChars.includes(char)) {
        stackError(machineState);
      } else if (char === machineState.settings.expandClose) {
        stackClose(machineState, idx, false);
      } else if (char === machineState.settings.separatorChar) {
        machineState.state = State['stack.Init'];
      } else {
        stackError(machineState);
      }
    }
  };

  function transform(content, options) {
    const machineState = {
      context: {
        matches: [],
        stack: [],
        variantStartIdx: 0,
        variantEndIdx: 0,
        content
      },
      state: State.init,
      settings: {
        badChars: [...badChars, ...whitespaceChars],
        expandClose: (options == null ? void 0 : options.expandCloseChar) ?? ')',
        expandOpen: (options == null ? void 0 : options.expandOpenChar) ?? '(',
        separatorChar: (options == null ? void 0 : options.separatorChar) ?? ',',
        variantChar: (options == null ? void 0 : options.variantChar) ?? ':'
      }
    };

    for (let idx = 0; idx < content.length; ++idx) {
      // eslint-disable-next-line @typescript-eslint/no-non-null-assertion
      const char = content[idx];
      const handleIdxForState = transformMachine[machineState.state];
      handleIdxForState(idx, char, machineState);
    }

    const matches = machineState.context.matches;

    if (matches.length) {
      let prevStart = 0;
      const str = matches.reduce((prev, cur) => {
        const substr = `${prev}${content.substring(prevStart, cur.start)}${cur.content}`;
        prevStart = cur.end + 1;
        return substr;
      }, '');
      return `${str}${content.substring(prevStart)}`;
    }

    return content;
  }

  function createTransformer(options) {
    return content => transform(content, options);
  }

  return createTransformer;

}));
//# sourceMappingURL=tailwind-group-variant.umd.js.map
